
//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include "c8051f320.h"        
#include "USB_API.h"

//-----------------------------------------------------------------------------
// 16-bit SFR Definitions for 'F32x
//-----------------------------------------------------------------------------
sfr16 TMR2RL = 0xCA;                   // Timer2 reload value
sfr16 TMR2 = 0xCC;                     // Timer2 counter
sfr16 ADC0 = 0xBD;                     // 10-bit ADC0 result


#define SYSCLK        24000000         // SYSCLK frequency in Hz   
#define ANALOG_INPUTS 8                // Number of AIN pins to measure
#define SAMPLES       20
#define INT_DEC       1                // Integrate and decimate ratio
#define TIMER0_RELOAD_HIGH  0          // Timer0 High register
#define TIMER0_RELOAD_LOW 255          // Timer0 Low register


bit USB_TX_READY = 1, USB_START = 0;                              
unsigned int accumulator[ANALOG_INPUTS];
BYTE Result_Output[16], counter = 0, i = 0;

// Port1 ADC Inputs
unsigned char idata PIN_TABLE[ANALOG_INPUTS] = {0,1,2,3,4,5,6,7};
unsigned char AMUX_INPUT = 0;          // Index of analog MUX inputs

//xdata BYTE OUT_BUFER[512];
BYTE In_Packet[1];   // Last packet received from host

/*** [BEGIN] USB Descriptor Information [BEGIN] ***/
code const UINT USB_VID = 0x10C4;
code const UINT USB_PID = 0xEA61;
code const BYTE USB_MfrStr[] = {0x1A,0x03,'S',0,'i',0,'l',0,'i',0,'c',0,'o',0,'n',0,' ',0,'L',0,'a',0,'b',0,'s',0};                      // Manufacturer String
code const BYTE USB_ProductStr[] = {0x10,0x03,'U',0,'S',0,'B',0,' ',0,'A',0,'P',0,'I',0}; //  Product Desc. String
code const BYTE USB_SerialStr[] = {0x0A,0x03,'N',0,'E',0,'I',0,'L',0};
code const BYTE USB_MaxPower = 150;           // Max current = 300 mA (15 * 2)
code const BYTE USB_PwAttributes = 0x80;      // Bus-powered, remote wakeup not supported
code const UINT USB_bcdDevice = 0x0100;       // Device release number 1.00
/*** [ END ] USB Descriptor Information [ END ] ***/




void Port_IO_Init()
{     
   	P1SKIP = 0xFF;                      // Skip all analog pins
    P1MDIN    = 0x00;
	//P0MDOUT   = 0xFF;
    XBR1      = 0xC0;
}

void Timer2_Init (void)
{
   TMR2CN = 0x00;                      // Stop Timer2; Clear TF2;
                                       // use SYSCLK as timebase, 16-bit
                                       // auto-reload
   CKCON |= 0x10;                      // Select SYSCLK for timer 2 source
   TMR2RL = 65535 - (SYSCLK / 12000);  // Init reload value for 10 us
   TMR2 = 0xffff;                      // Set to reload immediately
   ET2 = 1;                            // Enable Timer2 interrupts
   TR2 = 1;                            // Start Timer2
}

void ADC_Init()
{
   ADC0CN = 0x02;                      // ADC0 disabled, normal tracking,
                                       // conversion triggered on TMR2 overflow
   REF0CN = 0x08;                      // Enable internal VREF as VDD
   AMX0P = PIN_TABLE[0];               // ADC0 initial positive input = P1.0
   AMX0N = 0x1F;                       // ADC0 negative input = GND
                                       // i.e., single ended mode
   ADC0CF = ((SYSCLK/3000000)-1)<<3;   // Set SAR clock to 3MHz
   ADC0CF |= 0x00;                     // Right-justify results
   EIE1 |= 0x08;                       // Enable ADC0 EOC interrupt
   AD0EN = 1;                          // Enable ADC0
}

void Initialize(void)
{
    Port_IO_Init(); 
    Timer2_Init();
    ADC_Init();       
}



//-----------------------------------------------------------------------------
// Main Routine
//-----------------------------------------------------------------------------
void main(void) 
{
   PCA0MD &= ~0x40;                       // Disable Watchdog timer

   USB_Clock_Start();                     // Init USB clock *before* calling USB_Init
   USB_Init(USB_VID,USB_PID,USB_MfrStr,USB_ProductStr,USB_SerialStr,USB_MaxPower,USB_PwAttributes,USB_bcdDevice);
   Initialize();
   USB_Int_Enable();
   
   while (1);
}



void Timer2_ISR (void) interrupt 5
{
   TF2H = 0;                           // Clear Timer2 interrupt flag

   if (AMUX_INPUT == (ANALOG_INPUTS - 1))
   {
      AMX0P = PIN_TABLE[0];
   }
   else
   {
      AMX0P = PIN_TABLE[AMUX_INPUT+1];
   }
}


void ADC0_ISR (void) interrupt 10
{        
   AD0INT = 0;                         // Clear ADC conversion complete
                                       // overflow
 	   
   accumulator[AMUX_INPUT] = ADC0;                                  

   // Reset sample counter <int_dec> and <AMUX_INPUT> if the final input was
   // just read
   if(AMUX_INPUT == (ANALOG_INPUTS - 1))
   {
           if(USB_START == 1)
		   {
		    i = 0;
		    for(counter = 0; counter < 15; counter += 2)
			{
			   Result_Output[counter] = (accumulator[i]<<2)>>2;
			   Result_Output[counter+1] = accumulator[i]>>8;
			   i++;
			}
			Block_Write(Result_Output, 16);
		    USB_START = 0;
		   }
		  
      AMUX_INPUT = 0;                  // Reset input index back to P1.0
   }
   // Otherwise, increment the AMUX channel counter
   else
   {
      AMUX_INPUT++;                    // Step to the next analog mux input
   }
}





void USB_API_TEST_ISR(void) interrupt 16       // ISR for USB_API
{
	BYTE INTVAL = Get_Interrupt_Source();

	if (INTVAL & TX_COMPLETE)
	{
	    USB_TX_READY = 1;
	}

	if (INTVAL & RX_COMPLETE)
	{
		USB_START = 1;
		Block_Read(In_Packet, 1);
	}
		
	if (INTVAL & DEVICE_OPEN)
	{
		USB_START = 1;
	}
}

